@@ -1,7 +1,9 @@
-from __future__ import unicode_literals
+from __future__ import absolute_import, unicode_literals
 
 import os
 
+from mopidy import compat
+
 
 def path_to_data_dir(name):
     if not isinstance(name, bytes):
@@ -13,6 +15,7 @@ def path_to_data_dir(name):
 
 
 class IsA(object):
+
     def __init__(self, klass):
         self.klass = klass
 
@@ -20,7 +23,7 @@ class IsA(object):
         try:
             return isinstance(rhs, self.klass)
         except TypeError:
-            return type(rhs) == type(self.klass)
+            return type(rhs) == type(self.klass)  # flake8: noqa
 
     def __ne__(self, rhs):
         return not self.__eq__(rhs)
@@ -29,6 +32,6 @@ class IsA(object):
         return str(self.klass)
 
 
-any_int = IsA(int)
+any_int = IsA((int, long))
 any_str = IsA(str)
-any_unicode = IsA(unicode)
+any_unicode = IsA(compat.text_type)
