@@ -1,4 +1,4 @@
-from __future__ import unicode_literals
+from __future__ import absolute_import, unicode_literals
 
 from mopidy.models import Track
 
@@ -6,34 +6,37 @@ from tests.mpd import protocol
 
 
 class StatusHandlerTest(protocol.BaseTestCase):
+
     def test_clearerror(self):
-        self.sendRequest('clearerror')
+        self.send_request('clearerror')
         self.assertEqualResponse('ACK [0@0] {clearerror} Not implemented')
 
     def test_currentsong(self):
-        track = Track()
-        self.core.tracklist.add([track])
+        track = Track(uri='dummy:/a')
+        self.backend.library.dummy_library = [track]
+        self.core.tracklist.add(uris=[track.uri]).get()
+
         self.core.playback.play()
-        self.sendRequest('currentsong')
-        self.assertInResponse('file: ')
+        self.send_request('currentsong')
+        self.assertInResponse('file: dummy:/a')
         self.assertInResponse('Time: 0')
-        self.assertInResponse('Artist: ')
-        self.assertInResponse('Title: ')
-        self.assertInResponse('Album: ')
-        self.assertInResponse('Track: 0')
+        self.assertNotInResponse('Artist: ')
+        self.assertNotInResponse('Title: ')
+        self.assertNotInResponse('Album: ')
+        self.assertNotInResponse('Track: 0')
         self.assertNotInResponse('Date: ')
         self.assertInResponse('Pos: 0')
         self.assertInResponse('Id: 0')
         self.assertInResponse('OK')
 
     def test_currentsong_without_song(self):
-        self.sendRequest('currentsong')
+        self.send_request('currentsong')
         self.assertInResponse('OK')
 
     def test_stats_command(self):
-        self.sendRequest('stats')
+        self.send_request('stats')
         self.assertInResponse('OK')
 
     def test_status_command(self):
-        self.sendRequest('status')
+        self.send_request('status')
         self.assertInResponse('OK')
